@@ -1,24 +1,25 @@
 #!/usr/bin/env tclsh8.4
 # dpkgbuild.tcl
-# $Id: dpkgall.tcl 68752 2010-06-12 13:57:23Z jmr@macports.org $
+# $Id$
 #
+# Copyright (c) 2009-2011 The MacPorts Project
 # Copyright (c) 2004 Landon Fuller <landonf@macports.org>
 # Copyright (c) 2003 Kevin Van Vechten <kevin@opendarwin.org>
-# Copyright (c) 2002 Apple Computer, Inc.
+# Copyright (c) 2002 Apple Inc.
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions
 # are met:
 # 1. Redistributions of source code must retain the above copyright
-#    notice, this list of conditions and the following disclaimer.
+#	 notice, this list of conditions and the following disclaimer.
 # 2. Redistributions in binary form must reproduce the above copyright
-#    notice, this list of conditions and the following disclaimer in the
-#    documentation and/or other materials provided with the distribution.
-# 3. Neither the name of Apple Computer, Inc. nor the names of its contributors
-#    may be used to endorse or promote products derived from this software
-#    without specific prior written permission.
-# 
+#	 notice, this list of conditions and the following disclaimer in the
+#	 documentation and/or other materials provided with the distribution.
+# 3. Neither the name of Apple Inc. nor the names of its contributors
+#	 may be used to endorse or promote products derived from this software
+#	 without specific prior written permission.
+#
 # THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 # AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 # IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
@@ -87,27 +88,27 @@ namespace eval dpkg {
 
 # MacPorts UI Event Callbacks
 proc ui_prefix {priority} {
-    switch $priority {
-        debug {
-        	return "Debug: "
-        }
-        error {
-        	return "Error: "
-        }
-        warn {
-        	return "Warning: "
-        }
-        default {
-        	return ""
-        }
-    }
+	switch $priority {
+		debug {
+			return "Debug: "
+		}
+		error {
+			return "Error: "
+		}
+		warn {
+			return "Warning: "
+		}
+		default {
+			return ""
+		}
+	}
 }
 
 proc ui_channels {priority} {
 	global dpkg::logfd
-	if {[info exists logfd] && [string length $logfd] > 0 } {
+	if {[info exists logfd] && $logfd ne ""} {
 		return {$logfd}
-	} elseif {$message(priority) != "debug"} {
+	} elseif {$message(priority) ne "debug"} {
 		# If there's no log file, echo to stdout
 		return {stdout}
 	}
@@ -301,7 +302,7 @@ proc main {argc argv} {
 		exit 0
 	}
 
-	# We must have dpkg by now 
+	# We must have dpkg by now
 	if {[catch {set_architecture} result]} {
 		puts "$result."
 		puts "Have you initialized the build system? Use the -i flag:"
@@ -431,14 +432,17 @@ proc main {argc argv} {
 		array set portinfo [lindex $res 1]
 
 		if {![info exists portinfo(name)] ||
-			![info exists portinfo(version)] || 
-			![info exists portinfo(revision)] || 
+			![info exists portinfo(version)] ||
+			![info exists portinfo(revision)] ||
 			![info exists portinfo(categories)]} {
 			ui_noisy_error "Internal error: $name missing some portinfo keys"
 			close $logfd
 			continue
 		}
 
+		# open correct subport
+		set options(subport) $portinfo(name)
+
 		# Skip un-supported ports
 		if {[info exists portinfo(platforms)] && ${anyplatform_flag} != "true"} {
 			if {[lsearch $portinfo(platforms) $platformString] == -1} {
@@ -551,7 +555,7 @@ proc main {argc argv} {
 		}
 
 		ui_silent "Package build for $portinfo(name) succeeded"
-		
+
 		# Into the apt repository you go!
 		copy_pkg_to_apt $portinfo(name) $portinfo(version) $portinfo(revision) [lindex $portinfo(categories) 0]
 
@@ -593,7 +597,7 @@ proc main {argc argv} {
 			}
 		}
 
-		if {[info exists newpkglist]} {		
+		if {[info exists newpkglist]} {
 			set pkglist [lsort -unique [concat $newpkglist $pkglist]]
 		}
 
@@ -636,7 +640,7 @@ proc get_required_ports {args} {
 	set reqlist ""
 
 	foreach {binary port} $requiredports {
-		if {[find_binary $binary] == ""} {
+		if {[find_binary $binary] eq ""} {
 			lappend reqlist $port
 		}
 	}
@@ -681,7 +685,7 @@ proc initialize_system {args} {
 	set buildlist [lsort -unique $builddeps]
 
 	foreach port $builddeps {
-		if {[lsearch -exact $port $rundeps] >= 0 } {
+		if {[lsearch -exact $port $rundeps] >= 0} {
 			lappend removelist $port
 		}
 	}
@@ -690,6 +694,7 @@ proc initialize_system {args} {
 	set variations ""
 
 	foreach port [get_required_ports] {
+		set options(subport) $port
 		if {[catch {do_portexec $port [array get options] [array get variants] activate} result]} {
 			global errorInfo
 			ui_debug "$errorInfo"
@@ -704,7 +709,7 @@ proc initialize_system {args} {
 			set port [lindex $portlist 0]
 
 			ui_msg "Uninstalling $port."
-			if { [catch {registry_uninstall::uninstall $portname $portversion} result] } {
+			if { [catch {registry_uninstall::uninstall $portname $portversion "" 0 [array get options]} result] } {
 				global errorInfo
 				ui_debug "$errorInfo"
 				ui_noisy_errorr "Fatal error: Uninstalling $port failed: $result"
@@ -713,7 +718,7 @@ proc initialize_system {args} {
 		}
 		ui_msg "Done."
 	}
-			
+
 
 	if {[catch {set_architecture} result]} {
 		puts "Fatal error: $result."
@@ -853,14 +858,14 @@ proc get_dependencies {portname {includeBuildDeps "true"}} {
 
 proc get_dependencies_recurse {portname includeBuildDeps} {
 	set result {}
-	
+
 	set res [get_portinfo $portname]
 
 	foreach {name array} $res {
 		array set portinfo $array
 		if {![info exists portinfo(name)] ||
-			![info exists portinfo(version)] || 
-			![info exists portinfo(revision)] || 
+			![info exists portinfo(version)] ||
+			![info exists portinfo(revision)] ||
 			![info exists portinfo(categories)]} {
 			ui_error "Internal error: $name missing some portinfo keys"
 			continue
@@ -872,13 +877,13 @@ proc get_dependencies_recurse {portname includeBuildDeps} {
 		set depends {}
 		if {[info exists portinfo(depends_run)]} { eval "lappend depends $portinfo(depends_run)" }
 		if {[info exists portinfo(depends_lib)]} { eval "lappend depends $portinfo(depends_lib)" }
-		if {$includeBuildDeps == "true" && [info exists portinfo(depends_build)]} { 
+		if {$includeBuildDeps == "true" && [info exists portinfo(depends_build)]} {
 			eval "lappend depends $portinfo(depends_build)"
 		}
-		if {$includeBuildDeps == "true" && [info exists portinfo(depends_fetch)]} { 
+		if {$includeBuildDeps == "true" && [info exists portinfo(depends_fetch)]} {
 			eval "lappend depends $portinfo(depends_fetch)"
 		}
-		if {$includeBuildDeps == "true" && [info exists portinfo(depends_extract)]} { 
+		if {$includeBuildDeps == "true" && [info exists portinfo(depends_extract)]} {
 			eval "lappend depends $portinfo(depends_extract)"
 		}
 		foreach depspec $depends {
@@ -908,7 +913,7 @@ proc install_binary_if_available {dep} {
 	} else {
 		set verstring ${portversion}
 	}
-	
+
 	set receiptdir [file join $portprefix var db receipts ${portname} ${verstring}]
 	set pkgpath [get_pkgpath ${portname} ${portversion} ${portrevision}]
 
