@@ -1,8 +1,8 @@
 # -*- coding: utf-8; mode: tcl; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- vim:fenc=utf-8:ft=tcl:et:sw=4:ts=4:sts=4
-# $Id: portarchivefetch.tcl 101354 2013-01-09 14:05:39Z jmr@macports.org $
+# $Id$
 #
 # Copyright (c) 2002 - 2003 Apple Inc.
-# Copyright (c) 2004 - 2012 The MacPorts Project
+# Copyright (c) 2004 - 2013 The MacPorts Project
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -93,7 +93,8 @@ proc portarchivefetch::filter_sites {} {
         if {$missing} {
             continue
         }
-        if {$portfetch::mirror_sites::archive_prefix($site) == $prefix &&
+        if {$portfetch::mirror_sites::sites($site) ne {} &&
+            $portfetch::mirror_sites::archive_prefix($site) == $prefix &&
             $portfetch::mirror_sites::archive_frameworks_dir($site) == $frameworks_dir &&
             $portfetch::mirror_sites::archive_applications_dir($site) == $applications_dir &&
             ![catch {archiveTypeIsSupported $portfetch::mirror_sites::archive_type($site)}]} {
@@ -103,7 +104,7 @@ proc portarchivefetch::filter_sites {} {
     }
 
     # check if porturl itself points to an archive
-    if {[file rootname [file tail $porturl]] == [file rootname [get_portimage_name]] && [file extension $porturl] != ""} {
+    if {[file rootname [file tail $porturl]] eq [file rootname [get_portimage_name]] && [file extension $porturl] ne ""} {
         lappend ret [string range $porturl 0 end-[string length [file tail $porturl]]]:[string range [file extension $porturl] 1 end]
         archive.subdir
     }
@@ -136,7 +137,9 @@ proc portarchivefetch::checkarchivefiles {urls} {
 # returns full path to mirror list file
 proc portarchivefetch::get_full_archive_sites_path {} {
     global archive_sites.listfile archive_sites.listpath porturl
-    return [getportresourcepath $porturl [file join ${archive_sites.listpath} ${archive_sites.listfile}]]
+    # look up archive sites only from this ports tree,
+    # do not fallback to the default
+    return [getportresourcepath $porturl [file join ${archive_sites.listpath} ${archive_sites.listfile}] no]
 }
 
 # Perform the full checksites/checkarchivefiles sequence.
@@ -152,10 +155,10 @@ proc portarchivefetch::checkfiles {urls} {
 # Perform a standard fetch, assembling fetch urls from
 # the listed url variable and associated archive file
 proc portarchivefetch::fetchfiles {args} {
-    global archivefetch.fulldestpath UI_PREFIX
-    global archivefetch.user archivefetch.password archivefetch.use_epsv \
-           archivefetch.ignore_sslcert
-    global portverbose ports_binary_only
+    global archivefetch.fulldestpath UI_PREFIX \
+           archivefetch.user archivefetch.password archivefetch.use_epsv \
+           archivefetch.ignore_sslcert \
+           portverbose ports_binary_only
     variable archivefetch_urls
     variable ::portfetch::urlmap
 
@@ -185,15 +188,19 @@ proc portarchivefetch::fetchfiles {args} {
     if {${archivefetch.ignore_sslcert} != "no"} {
         lappend fetch_options "--ignore-ssl-cert"
     }
-    if {$portverbose == "yes"} {
-        lappend fetch_options "-v"
+    if {$portverbose eq "yes"} {
+        lappend fetch_options "--progress"
+        lappend fetch_options "builtin"
+    } elseif {[llength [info commands ui_progress_download]] > 0} {
+        lappend fetch_options "--progress"
+        lappend fetch_options "ui_progress_download"
     }
     set sorted no
 
     set existing_archive [find_portarchive_path]
 
     foreach {url_var archive} $archivefetch_urls {
-        if {![file isfile ${archivefetch.fulldestpath}/${archive}] && $existing_archive == ""} {
+        if {![file isfile ${archivefetch.fulldestpath}/${archive}] && $existing_archive eq ""} {
             ui_info "$UI_PREFIX [format [msgcat::mc "%s doesn't seem to exist in %s"] $archive ${archivefetch.fulldestpath}]"
             if {![file writable ${archivefetch.fulldestpath}]} {
                 return -code error [format [msgcat::mc "%s must be writable"] ${archivefetch.fulldestpath}]
@@ -202,7 +209,7 @@ proc portarchivefetch::fetchfiles {args} {
                 return -code error [format [msgcat::mc "%s must be writable"] $incoming_path]
             }
             if {!$sorted} {
-                portfetch::sortsites archivefetch_urls {} archive_sites
+                portfetch::sortsites archivefetch_urls archive_sites
                 set sorted yes
             }
             if {![info exists urlmap($url_var)]} {
@@ -212,7 +219,7 @@ proc portarchivefetch::fetchfiles {args} {
             set failed_sites 0
             unset -nocomplain fetched
             foreach site $urlmap($url_var) {
-                if {[string index $site end] != "/"} {
+                if {[string index $site end] ne "/"} {
                     append site "/[option archive.subdir]"
                 } else {
                     append site [option archive.subdir]
@@ -253,14 +260,16 @@ proc portarchivefetch::fetchfiles {args} {
                         ui_debug "openssl output: $result"
                     }
                 }
+                file delete -force $signature
                 if {!$verified} {
-                    return -code error "Failed to verify signature for archive!"
-                }
-                if {[catch {file rename -force "${incoming_path}/${archive}.TMP" "${archivefetch.fulldestpath}/${archive}"} result]} {
+                    # fall back to building from source (or error out later if binary only mode)
+                    ui_warn "Failed to verify signature for archive!"
+                    file delete -force "${incoming_path}/${archive}.TMP"
+                    break
+                } elseif {[catch {file rename -force "${incoming_path}/${archive}.TMP" "${archivefetch.fulldestpath}/${archive}"} result]} {
                     ui_debug "$::errorInfo"
                     return -code error "Failed to move downloaded archive into place: $result"
                 }
-                file delete -force $signature
                 set archive_exists 1
                 break
             }
@@ -277,7 +286,7 @@ proc portarchivefetch::fetchfiles {args} {
         }
         return 0
     }
-    if {[info exists ports_binary_only] && $ports_binary_only == "yes"} {
+    if {([info exists ports_binary_only] && $ports_binary_only eq "yes") || [_archive_available]} {
         return -code error "archivefetch failed for [option subport] @[option version]_[option revision][option portvariants]"
     } else {
         return 0
@@ -300,6 +309,7 @@ proc portarchivefetch::archivefetch_start {args} {
     if {[info exists all_archive_files] && [llength $all_archive_files] > 0} {
         ui_msg "$UI_PREFIX [format [msgcat::mc "Fetching archive for %s"] $subport]"
     }
+    portfetch::check_dns
 }
 
 # Main archive fetch routine
