@@ -1,8 +1,9 @@
 # et:ts=4
 # portrpm.tcl
-# $Id: portrpm.tcl 51521 2009-05-27 08:48:05Z jmr@macports.org $
+# $Id$
 #
-# Copyright (c) 2002 - 2003 Apple Computer, Inc.
+# Copyright (c) 2005 - 2007, 2009 - 2011, 2013 The MacPorts Project
+# Copyright (c) 2002 - 2003 Apple Inc.
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -13,10 +14,10 @@
 # 2. Redistributions in binary form must reproduce the above copyright
 #    notice, this list of conditions and the following disclaimer in the
 #    documentation and/or other materials provided with the distribution.
-# 3. Neither the name of Apple Computer, Inc. nor the names of its contributors
+# 3. Neither the name of Apple Inc. nor the names of its contributors
 #    may be used to endorse or promote products derived from this software
 #    without specific prior written permission.
-# 
+#
 # THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 # AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 # IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
@@ -36,29 +37,40 @@ package require portutil 1.0
 set org.macports.rpm [target_new org.macports.rpm portrpm::rpm_main]
 target_runtype ${org.macports.rpm} always
 target_provides ${org.macports.rpm} rpm
-target_requires ${org.macports.rpm} destroot
+target_requires ${org.macports.rpm} archivefetch unarchive destroot
 
 namespace eval portrpm {
 }
 
-options package.destpath
+# Options
+options rpm.asroot \
+        package.destpath
+
+# Set up defaults
+default rpm.asroot yes
+
+default rpm.srcdir {${prefix}/src/macports}
+default rpm.tmpdir {${prefix}/var/tmp}
 
 set_ui_prefix
 
 proc portrpm::rpm_main {args} {
-    global name version revision UI_PREFIX
-    
-    ui_msg "$UI_PREFIX [format [msgcat::mc "Creating RPM package for %s-%s"] ${name} ${version}]"
-    
-    return [rpm_pkg $name $version $revision]
+    global subport version revision UI_PREFIX
+
+    ui_msg "$UI_PREFIX [format [msgcat::mc "Creating RPM package for %s-%s"] ${subport} ${version}]"
+
+    return [rpm_pkg $subport $version $revision]
 }
 
 proc portrpm::rpm_pkg {portname portversion portrevision} {
-    global UI_PREFIX package.destpath portdbpath destpath workpath prefix categories maintainers description long_description homepage epoch portpath
-	global os.platform os.arch os.version os.major
-    
+    global UI_PREFIX rpm.asroot package.destpath portdbpath destpath workpath \
+           prefix categories maintainers description long_description \
+           homepage epoch portpath os.platform os.arch os.version os.major \
+           supported_archs configure.build_arch license
+
     set rpmdestpath ""
     if {![string equal ${package.destpath} ${workpath}] && ![string equal ${package.destpath} ""]} {
+        set rpm.asroot no
         set pkgpath ${package.destpath}
         file mkdir ${pkgpath}/BUILD \
                    ${pkgpath}/RPMS \
@@ -67,26 +79,27 @@ proc portrpm::rpm_pkg {portname portversion portrevision} {
                    ${pkgpath}/SRPMS
         set rpmdestpath "--define '_topdir ${pkgpath}'"
     }
-    
+
     set rpmbuildarch ""
-    if {[variant_isset "universal"]} {
-        set rpmbuildarch "--target fat"
-    }
-    if {false} {
+    if {$supported_archs eq "noarch"} {
         set rpmbuildarch "--target noarch"
+    } elseif {[variant_exists universal] && [variant_isset universal]} {
+        set rpmbuildarch "--target fat"
+    } elseif {${configure.build_arch} != ""} {
+        set rpmbuildarch "--target ${configure.build_arch}"
     }
-    
+
     foreach dir [list "${prefix}/src/macports/RPMS" "${prefix}/src/apple/RPMS" "/usr/src/apple/RPMS" "/macports/rpms/RPMS"] {
-        foreach arch {"ppc" "i386" "fat" "noarch"} {
+        foreach arch [list ${configure.build_arch} ${os.arch} "fat" "noarch"] {
             set rpmpath "$dir/${arch}/${portname}-${portversion}-${portrevision}.${arch}.rpm"
 	    if {[file readable $rpmpath] && ([file mtime ${rpmpath}] >= [file mtime ${portpath}/Portfile])} {
                 ui_debug "$rpmpath"
-                ui_msg "$UI_PREFIX [format [msgcat::mc "RPM package for %s-%s is up-to-date"] ${portname} ${portversion}]"
+                ui_msg "$UI_PREFIX [format [msgcat::mc "RPM package for %s version %s is up-to-date"] ${portname} ${portversion}]"
                 return 0
             }
         }
     }
-    
+
     set specpath ${workpath}/${portname}.spec
     # long_description, description, or homepage may not exist
     foreach variable {long_description description homepage categories maintainers} {
@@ -97,9 +110,8 @@ proc portrpm::rpm_pkg {portname portversion portrevision} {
         }
     }
     set category   [lindex [split $categories " "] 0]
-    set license    "Unknown"
     set maintainer $maintainers
-    
+
     set dependencies {}
     # get deplist
     set deps [make_dependency_list $portname]
@@ -115,7 +127,7 @@ proc portrpm::rpm_pkg {portname portversion portrevision} {
 
 	# depend on system (virtual packages for apple stuff)
 	lappend dependencies "org.macports.${os.platform}${os.major}"
-    
+
     set listpath ${workpath}/${portname}.filelist
     system "rm -f '${workpath}/${portname}.filelist' && touch '${workpath}/${portname}.filelist'"
     #system "cd '${destpath}' && find . -type d | grep -v -E '^.$' | sed -e 's/\"/\\\"/g' -e 's/^./%dir \"/' -e 's/$/\"/' > '${workpath}/${portname}.filelist'"
@@ -123,7 +135,7 @@ proc portrpm::rpm_pkg {portname portversion portrevision} {
     system "cd '${destpath}' && find . ! -type d | grep /etc/ | sed -e 's/\"/\\\"/g' -e 's/^./%config \"/' -e 's/$/\"/' >> '${workpath}/${portname}.filelist'"
     write_spec ${specpath} ${destpath} ${listpath} $portname $portversion $portrevision $pkg_description $pkg_long_description $pkg_homepage $category $license $maintainer $dependencies $epoch
     system "MP_USERECEIPTS='${portdbpath}/receipts' rpmbuild -bb -v ${rpmbuildarch} ${rpmdestpath} ${specpath}"
-    
+
     return 0
 }
 
@@ -137,17 +149,17 @@ proc portrpm::make_dependency_list {portname} {
     }
     foreach {name array} $res {
         array set portinfo $array
-	
+
         if {[info exists portinfo(depends_run)] || [info exists portinfo(depends_lib)]} {
             # get the union of depends_run and depends_lib
             # xxx: only examines the portfile component of the depspec
             set depends {}
             if {[info exists portinfo(depends_run)]} { eval "lappend depends $portinfo(depends_run)" }
             if {[info exists portinfo(depends_lib)]} { eval "lappend depends $portinfo(depends_lib)" }
-	    
+
             foreach depspec $depends {
                 set dep [lindex [split $depspec :] end]
-		
+
                 # xxx: nasty hack
                 if {$dep != "XFree86"} {
                     eval "lappend result [make_dependency_list $dep]"
@@ -215,7 +227,7 @@ License: ${license}
 URL: ${homepage}
 BuildRoot: ${destroot}
 AutoReq: no"
-    if {[expr ${epoch} != 0]} {
+    if {$epoch != 0} {
 	    puts $specfd "Epoch: ${epoch}"
     }
     if {[llength ${dependencies}] != 0} {
