@@ -1,8 +1,9 @@
 # et:ts=4
 # portsrpm.tcl
-# $Id: portsrpm.tcl 42662 2008-11-28 23:18:50Z raimue@macports.org $
+# $Id$
 #
-# Copyright (c) 2002 - 2003 Apple Computer, Inc.
+# Copyright (c) 2007, 2009, 2011, 2013 The MacPorts Project
+# Copyright (c) 2002 - 2003 Apple Inc.
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -13,10 +14,10 @@
 # 2. Redistributions in binary form must reproduce the above copyright
 #    notice, this list of conditions and the following disclaimer in the
 #    documentation and/or other materials provided with the distribution.
-# 3. Neither the name of Apple Computer, Inc. nor the names of its contributors
+# 3. Neither the name of Apple Inc. nor the names of its contributors
 #    may be used to endorse or promote products derived from this software
 #    without specific prior written permission.
-# 
+#
 # THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 # AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 # IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
@@ -34,49 +35,60 @@ package provide portsrpm 1.0
 package require portutil 1.0
 package require portfetch 1.0
 
-set org.macports.srpm [target_new org.macports.srpm srpm_main]
+set org.macports.srpm [target_new org.macports.srpm portsrpm::srpm_main]
 target_runtype ${org.macports.srpm} always
 target_provides ${org.macports.srpm} srpm
 target_requires ${org.macports.srpm} checksum
 
+namespace eval portsrpm {
+}
+
 options package.destpath
 
+# Set up defaults
+default srpm.asroot yes
+
 set_ui_prefix
 
-proc srpm_main {args} {
-    global portname portversion portrevision UI_PREFIX
-    
-    ui_msg "$UI_PREFIX [format [msgcat::mc "Creating SRPM package for %s-%s"] ${portname} ${portversion}]"
-    
-    return [srpm_pkg $portname $portversion $portrevision]
+proc portsrpm::srpm_main {args} {
+    global subport version revision UI_PREFIX
+
+    ui_msg "$UI_PREFIX [format [msgcat::mc "Creating SRPM package for %s-%s"] ${subport} ${version}]"
+
+    return [srpm_pkg $subport $version $revision]
 }
 
-proc srpm_pkg {portname portversion portrevision} {
-    global UI_PREFIX package.destpath portdbpath destpath workpath distpath prefix categories maintainers description long_description homepage epoch portpath distfiles fetch_urls
-	global os.platform os.arch os.version os.major
-    
+proc portsrpm::srpm_pkg {portname portversion portrevision} {
+    global UI_PREFIX package.destpath portdbpath destpath workpath distpath \
+           prefix categories maintainers description long_description \
+           homepage epoch portpath distfiles os.platform os.arch os.version \
+           os.major
+
+    set fetch_urls {}
+    portfetch::checkfiles fetch_urls
+
     set rpmdestpath ""
     if {![string equal ${package.destpath} ${workpath}] && ![string equal ${package.destpath} ""]} {
         set pkgpath ${package.destpath}
-        system "mkdir -p ${pkgpath}/BUILD"
-        system "mkdir -p ${pkgpath}/RPMS"
-        system "mkdir -p ${pkgpath}/SOURCES"
-        system "mkdir -p ${pkgpath}/SPECS"
-        system "mkdir -p ${pkgpath}/SRPMS"
+        file mkdir ${pkgpath}/BUILD \
+                   ${pkgpath}/RPMS \
+                   ${pkgpath}/SOURCES \
+                   ${pkgpath}/SPECS \
+                   ${pkgpath}/SRPMS
         set rpmdestpath "--define '_topdir ${pkgpath}'"
     }
-    
+
     foreach dir [list "${prefix}/src/macports/SRPMS" "${prefix}/src/apple/SRPMS" "/usr/src/apple/SRPMS" "/macports/rpms/SRPMS"] {
         foreach arch {"src" "nosrc"} {
             set rpmpath "$dir/${portname}-${portversion}-${portrevision}.${arch}.rpm"
 	    if {[file readable $rpmpath] && ([file mtime ${rpmpath}] >= [file mtime ${portpath}/Portfile])} {
                 ui_debug "$rpmpath"
-                ui_msg "$UI_PREFIX [format [msgcat::mc "SRPM package for %s-%s is up-to-date"] ${portname} ${portversion}]"
+                ui_msg "$UI_PREFIX [format [msgcat::mc "SRPM package for %s version %s is up-to-date"] ${portname} ${portversion}]"
                 return 0
             }
         }
     }
-    
+
     set specpath ${workpath}/${portname}-port.spec
     # long_description, description, or homepage may not exist
     foreach variable {long_description description homepage categories maintainers} {
@@ -89,7 +101,7 @@ proc srpm_pkg {portname portversion portrevision} {
     set category   [lindex [split $categories " "] 0]
     set license    "Unknown"
     set maintainer $maintainers
-    
+
     set dependencies {}
     # get deplist
     set deps [make_dependency_list $portname]
@@ -110,38 +122,46 @@ proc srpm_pkg {portname portversion portrevision} {
     set sourcespath "`rpm --eval %{_sourcedir}`"
 
     system "cp -p ${portpath}/Portfile ${sourcespath}/$portname-Portfile"
-    system "cd ${portpath} && zip -r -q ${sourcespath}/$portname-files.zip files -x \\*.DS_Store -x files/.svn\\*"
+    if {[info exists ${portpath}/files]} {
+        system "cd ${portpath} && zip -r -q ${sourcespath}/$portname-files.zip files -x \\*.DS_Store -x files/.svn\\*"
+        set zip $portname-files.zip
+    } else {
+        set zip ""
+    }
     foreach dist $distfiles {
         system "cp -p ${distpath}/${dist} ${sourcespath}/${dist}"
     }
-    
-    write_port_spec ${specpath} $portname $portversion $portrevision $pkg_description $pkg_long_description $pkg_homepage $category $license $maintainer $distfiles $fetch_urls $dependencies $epoch $src
+
+    write_port_spec ${specpath} $portname $portversion $portrevision $pkg_description $pkg_long_description $pkg_homepage $category $license $maintainer $distfiles $fetch_urls $dependencies $epoch $src $zip
     system "rpmbuild -bs -v --nodeps ${rpmdestpath} ${specpath}"
-    
+
     return 0
 }
 
-proc make_dependency_list {portname} {
+proc portsrpm::make_dependency_list {portname} {
     set result {}
-    if {[catch {set res [mport_search "^$portname\$"]} error]} {
+    if {[catch {set res [mport_lookup $portname]} error]} {
 		global errorInfo
 		ui_debug "$errorInfo"
-        ui_error "port search failed: $error"
+        ui_error "port lookup failed: $error"
         return 1
     }
     foreach {name array} $res {
         array set portinfo $array
-	
-        if {[info exists portinfo(depends_build)] || [info exists portinfo(depends_lib)]} {
-            # get the union of depends_build and depends_lib
+
+        if {[info exists portinfo(depends_fetch)] || [info exists portinfo(depends_extract)]
+            || [info exists portinfo(depends_build)] || [info exists portinfo(depends_lib)]} {
+            # get the union of depends_fetch, depends_extract, depends_build and depends_lib
             # xxx: only examines the portfile component of the depspec
             set depends {}
+            if {[info exists portinfo(depends_fetch)]} { eval "lappend depends $portinfo(depends_fetch)" }
+            if {[info exists portinfo(depends_extract)]} { eval "lappend depends $portinfo(depends_extract)" }
             if {[info exists portinfo(depends_build)]} { eval "lappend depends $portinfo(depends_build)" }
             if {[info exists portinfo(depends_lib)]} { eval "lappend depends $portinfo(depends_lib)" }
-	    
+
             foreach depspec $depends {
                 set dep [lindex [split $depspec :] end]
-		
+
                 # xxx: nasty hack
                 if {$dep != "XFree86"} {
                     eval "lappend result [make_dependency_list $dep]"
@@ -155,7 +175,7 @@ proc make_dependency_list {portname} {
     return $result
 }
 
-proc word_wrap {orig Length} {
+proc portsrpm::word_wrap {orig Length} {
     set pos 0
     set line ""
     set text ""
@@ -185,7 +205,7 @@ proc word_wrap {orig Length} {
     return $text
 }
 
-proc write_port_spec {specfile portname portversion portrevision description long_description homepage category license maintainer distfiles fetch_urls dependencies epoch src} {
+proc portsrpm::write_port_spec {specfile portname portversion portrevision description long_description homepage category license maintainer distfiles fetch_urls dependencies epoch src zip} {
     set specfd [open ${specfile} w+]
     set origportname ${portname}
     regsub -all -- "\-" $portversion "_" portversion
@@ -203,21 +223,23 @@ Group: ${category}
 License: ${license}
 URL: ${homepage}
 BuildRoot: %{_tmppath}/%{name}-%{version}-root
-Source0: ${portname}-Portfile
-Source1: ${portname}-files.zip"
-    if {[expr ${epoch} != 0]} {
+Source0: ${portname}-Portfile"
+    if {$zip ne ""} {
+        puts $specfd "Source1: $zip"
+    }
+    if {$epoch != 0} {
 	    puts $specfd "Epoch: ${epoch}"
     }
     set first 2
     set count $first
     puts $specfd "#distfiles"
     foreach file ${distfiles} {
-    
+
         puts -nonewline $specfd "Source${count}: "
         if {![info exists $fetch_urls]} {
         foreach {url_var distfile}  ${fetch_urls} {
-            if {[string equal $distfile $file]} {
-                 global portfetch::$url_var
+            if {$distfile eq $file} {
+                 global portfetch::$url_var master_sites
                  set site [lindex [set $url_var] 0]
                  set file [portfetch::assemble_url $site $distfile]
                  break
@@ -237,12 +259,17 @@ Source1: ${portname}-files.zip"
 	}
     }
     set wrap_description [word_wrap ${long_description} 72]
+    if {$zip ne ""} {
+        set and "-a 1"
+    } else {
+        set and ""
+    }
     puts $specfd "
 %description
 $wrap_description
 
 %prep
-%setup -c -a 1 -T
+%setup -c $and -T
 cp -p %{SOURCE0} Portfile
 #prepare work area
 port fetch
