@@ -1,6 +1,6 @@
 # -*- coding: utf-8; mode: tcl; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- vim:fenc=utf-8:ft=tcl:et:sw=4:ts=4:sts=4
 # portunarchive.tcl
-# $Id: portunarchive.tcl 90958 2012-03-19 18:14:31Z jmr@macports.org $
+# $Id$
 #
 # Copyright (c) 2005, 2007-2012 The MacPorts Project
 # Copyright (c) 2004 Robert Shaw <rshaw@opendarwin.org>
@@ -18,7 +18,7 @@
 # 3. Neither the name of Apple Inc. nor the names of its contributors
 #    may be used to endorse or promote products derived from this software
 #    without specific prior written permission.
-# 
+#
 # THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 # AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 # IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
@@ -71,14 +71,14 @@ proc portunarchive::unarchive_init {args} {
     set skipped 0
     if {[check_statefile target org.macports.unarchive $target_state_fd]} {
         return 0
-    } elseif {[info exists ports_source_only] && $ports_source_only == "yes"} {
+    } elseif {[info exists ports_source_only] && $ports_source_only eq "yes"} {
         ui_debug "Skipping unarchive ($subport) since source-only is set"
         set skipped 1
     } elseif {[check_statefile target org.macports.destroot $target_state_fd]
               && [file isdirectory $destroot]} {
         ui_debug "Skipping unarchive ($subport) since destroot completed"
         set skipped 1
-    } elseif {[info exists ports_force] && $ports_force == "yes"} {
+    } elseif {[info exists ports_force] && $ports_force eq "yes"} {
         ui_debug "Skipping unarchive ($subport) since force is set"
         set skipped 1
     } else {
@@ -88,7 +88,7 @@ proc portunarchive::unarchive_init {args} {
         if {${unarchive.path} != ""} {
             ui_debug "Found [string toupper ${unarchive.type}] archive: ${unarchive.path}"
         } else {
-            if {[info exists ports_binary_only] && $ports_binary_only == "yes"} {
+            if {[info exists ports_binary_only] && $ports_binary_only eq "yes"} {
                 return -code error "Archive for ${subport} ${version}_${revision}${portvariants} not found, required when binary-only is set!"
             } else {
                 ui_debug "Skipping unarchive ($subport) since no suitable archive found"
@@ -201,7 +201,7 @@ proc portunarchive::unarchive_command_setup {args} {
                 return -code error "No '$tar' was found on this system!"
             }
         }
-        xar|xpkg {
+        xar {
             set xar "xar"
             if {[catch {set xar [findBinary $xar ${portutil::autoconf::xar_path}]} errmsg] == 0} {
                 ui_debug "Using $xar"
@@ -278,6 +278,7 @@ proc portunarchive::unarchive_finish {args} {
         file copy -force $plus_state $statefile
         file mtime $statefile [clock seconds]
         chownAsRoot $statefile
+        update_statefile checksum [sha256 file [option portpath]/Portfile] $statefile
         set newstate 1
     } else {
         # fake it
