@@ -1,8 +1,9 @@
 # -*- coding: utf-8; mode: tcl; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- vim:fenc=utf-8:filetype=tcl:et:sw=4:ts=4:sts=4
 # portbuild.tcl
-# $Id: portbuild.tcl 55280 2009-08-08 02:45:30Z jmr@macports.org $
+# $Id$
 #
-# Copyright (c) 2002 - 2003 Apple Computer, Inc.
+# Copyright (c) 2007 - 2013 The MacPorts Project
+# Copyright (c) 2002 - 2004 Apple Inc.
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -13,7 +14,7 @@
 # 2. Redistributions in binary form must reproduce the above copyright
 #    notice, this list of conditions and the following disclaimer in the
 #    documentation and/or other materials provided with the distribution.
-# 3. Neither the name of Apple Computer, Inc. nor the names of its contributors
+# 3. Neither the name of Apple Inc. nor the names of its contributors
 #    may be used to endorse or promote products derived from this software
 #    without specific prior written permission.
 #
@@ -35,65 +36,90 @@ package require portutil 1.0
 
 set org.macports.build [target_new org.macports.build portbuild::build_main]
 target_provides ${org.macports.build} build
-target_requires ${org.macports.build} main fetch extract checksum patch configure
+target_requires ${org.macports.build} main fetch checksum extract patch configure
 target_prerun ${org.macports.build} portbuild::build_start
 
 namespace eval portbuild {
 }
 
 # define options
-options build.target
-options build.nice
-options build.jobs
-options build.asroot
-options use_parallel_build
+options build.asroot \
+        build.jobs \
+        build.target \
+        use_parallel_build
 commands build
 # defaults
 default build.asroot no
-default build.dir {${workpath}/${worksrcdir}}
+default build.dir {${worksrcpath}}
 default build.cmd {[portbuild::build_getmaketype]}
 default build.nice {${buildnicevalue}}
 default build.jobs {[portbuild::build_getjobs]}
-default build.pre_args {${build.target}}
+default build.pre_args {[portbuild::build_getargs]}
 default build.target "all"
+default build.type "default"
 default use_parallel_build yes
 
 set_ui_prefix
 
+# Automatically called from macports1.0 after evaluating the Portfile. If
+# ${build.type} == bsd, ensures bsdmake is present by adding a bin:-style
+# dependency.
+proc portbuild::add_automatic_buildsystem_dependencies {} {
+    global build.type.add_deps
+    if {!${build.type.add_deps}} {
+        return
+    }
+    if {[option build.type] eq "bsd" && [option os.platform] eq "darwin"} {
+        ui_debug "build.type is BSD, adding bin:bsdmake:bsdmake build dependency"
+        depends_build-delete bin:bsdmake:bsdmake
+        depends_build-append bin:bsdmake:bsdmake
+    }
+    if {[option build.type] eq "gnu" && [option os.platform] eq "freebsd"} {
+        ui_debug "build.type is GNU, adding bin:gmake:gmake build dependency"
+        depends_build-delete bin:gmake:gmake
+        depends_build-append bin:gmake:gmake
+    }
+}
+# Register the above procedure as a callback after Portfile evaluation
+port::register_callback portbuild::add_automatic_buildsystem_dependencies
+# and an option to turn it off if required
+options build.type.add_deps
+default build.type.add_deps yes
+
 proc portbuild::build_getmaketype {args} {
-    if {![exists build.type]} {
+    if {[option build.type] eq "default"} {
         return [findBinary make $portutil::autoconf::make_path]
     }
     switch -exact -- [option build.type] {
         bsd {
-            if {[option os.platform] == "darwin"} {
+            if {[option os.platform] eq "darwin"} {
                 return [findBinary bsdmake $portutil::autoconf::bsdmake_path]
-            } elseif {[option os.platform] == "freebsd"} {
+            } elseif {[option os.platform] eq "freebsd"} {
                 return [findBinary make $portutil::autoconf::make_path]
             } else {
                 return [findBinary pmake $portutil::autoconf::bsdmake_path]
             }
         }
         gnu {
-            if {[option os.platform] == "darwin"} {
+            if {[option os.platform] eq "darwin"} {
                 return [findBinary gnumake $portutil::autoconf::gnumake_path]
-            } elseif {[option os.platform] == "linux"} {
+            } elseif {[option os.platform] eq "linux"} {
                 return [findBinary make $portutil::autoconf::make_path]
             } else {
                 return [findBinary gmake $portutil::autoconf::gnumake_path]
             }
         }
-        pbx {
-            if {[option os.platform] != "darwin"} {
-                return -code error "[format [msgcat::mc "This port requires 'pbxbuild/xcodebuild', which is not available on %s."] [option os.platform]]"
+        pbx -
+        xcode {
+            if {[option os.platform] ne "darwin"} {
+                return -code error "[format [msgcat::mc "This port requires 'xcodebuild', which is not available on %s."] [option os.platform]]"
             }
 
-            if {[catch {set xcodebuild [binaryInPath xcodebuild]}] == 0} {
-                return $xcodebuild
-            } elseif {[catch {set pbxbuild [binaryInPath pbxbuild]}] == 0} {
-                return $pbxbuild
+            global xcodebuildcmd
+            if {$xcodebuildcmd ne "none"} {
+                return $xcodebuildcmd
             } else {
-                return -code error "Neither pbxbuild nor xcodebuild were found on this system!"
+                return -code error "xcodebuild was not found on this system!"
             }
         }
         default {
@@ -103,17 +129,6 @@ proc portbuild::build_getmaketype {args} {
     }
 }
 
-proc portbuild::build_getnicevalue {args} {
-    if {![exists build.nice] || [string match "* *" [option build.cmd]]} {
-        return ""
-    }
-    set nice [option build.nice]
-    if {![string is integer -strict $nice] || $nice <= 0} {
-        return ""
-    }
-    return "[findBinary nice $portutil::autoconf::nice_path] -n $nice "
-}
-
 proc portbuild::build_getjobs {args} {
     global buildmakejobs
     set jobs $buildmakejobs
@@ -124,8 +139,8 @@ proc portbuild::build_getjobs {args} {
             ui_warn "failed to determine the number of available CPUs (probably not supported on this platform)"
             ui_warn "defaulting to $jobs jobs, consider setting buildmakejobs to a nonzero value in macports.conf"
         }
-        if {$jobs > $memsize / 1000000000 + 1} {
-            set jobs [expr $memsize / 1000000000 + 1]
+        if {[info exists memsize] && $jobs > $memsize / 1000000000 + 1} {
+            set jobs [expr {$memsize / 1000000000 + 1}]
         }
     }
     if {![string is integer -strict $jobs] || $jobs <= 1} {
@@ -134,6 +149,17 @@ proc portbuild::build_getjobs {args} {
     return $jobs
 }
 
+proc portbuild::build_getargs {args} {
+    if {(([option build.type] eq "default" && [option os.platform] ne "freebsd") || \
+         ([option build.type] eq "gnu")) \
+        && [regexp "^(/\\S+/|)(g|gnu|)make(\\s+.*|)$" [option build.cmd]]} {
+        # Print "Entering directory" lines for better log debugging
+        return "-w [option build.target]"
+    }
+
+    return "[option build.target]"
+}
+
 proc portbuild::build_getjobsarg {args} {
     # check if port allows a parallel build
     global use_parallel_build
@@ -145,23 +171,26 @@ proc portbuild::build_getjobsarg {args} {
     if {![exists build.jobs] || !([string match "*make*" [option build.cmd]] || [string match "*scons*" [option build.cmd]])} {
         return ""
     }
-    return " -j[option build.jobs]"
+    set jobs [option build.jobs]
+    if {![string is integer -strict $jobs] || $jobs <= 1} {
+        return ""
+    }
+    return " -j$jobs"
 }
 
 proc portbuild::build_start {args} {
     global UI_PREFIX
 
-    ui_msg "$UI_PREFIX [format [msgcat::mc "Building %s"] [option name]]"
+    ui_notice "$UI_PREFIX [format [msgcat::mc "Building %s"] [option subport]]"
 }
 
 proc portbuild::build_main {args} {
     global build.cmd
 
-    set nice_prefix [build_getnicevalue]
     set jobs_suffix [build_getjobsarg]
 
     set realcmd ${build.cmd}
-    set build.cmd "$nice_prefix${build.cmd}$jobs_suffix"
+    set build.cmd "${build.cmd}$jobs_suffix"
     command_exec build
     set build.cmd ${realcmd}
     return 0
