@@ -1,6 +1,6 @@
 # et:ts=4
 # portlint.tcl
-# $Id: portlint.tcl 92731 2012-05-05 03:31:54Z jmr@macports.org $
+# $Id$
 
 package provide portlint 1.0
 package require portutil 1.0
@@ -138,7 +138,7 @@ proc portlint::lint_main {args} {
             if {$nitpick} {
                 seek $f -1 end
                 set last [read $f 1]
-                if {![string match "\n" $last]} {
+                if {"\n" ne $last} {
                     ui_warn "Line $lineno has missing newline (at end of file)"
                     incr warnings
                 }
@@ -153,12 +153,12 @@ proc portlint::lint_main {args} {
             incr errors
         }
 
-        if {($require_after == "PortSystem" || $require_after == "PortGroup") && \
+        if {($require_after eq "PortSystem" || $require_after eq "PortGroup") && \
             [string match "PortGroup*" $line]} {
             set require_blank false
         }
 
-        if {$nitpick && $require_blank && ($line != "")} {
+        if {$nitpick && $require_blank && ($line ne "")} {
             ui_warn "Line $lineno should be a newline (after $require_after)"
             incr warnings
         }
@@ -174,13 +174,10 @@ proc portlint::lint_main {args} {
             ui_info "OK: Line $lineno has emacs/vim Mode"
             incr topline_number
         }
-        if {($lineno == $topline_number) && ![string match "*\$Id*\$" $line]} {
-            ui_warn "Line $lineno is missing RCS tag (\$Id\$)"
+
+        if {[string match "*\$Id*\$" $line]} {
+            ui_warn "Line $lineno is using obsolete RCS tag (\$Id\$)"
             incr warnings
-        } elseif {($lineno == $topline_number)} {
-            ui_info "OK: Line $lineno has RCS tag (\$Id\$)"
-            set require_blank true
-            set require_after "RCS tag"
         }
         
         # skip the rest for comment lines (not perfectly accurate...)
@@ -191,7 +188,7 @@ proc portlint::lint_main {args} {
 
         if {[string match "PortSystem*" $line]} {
             if {$seen_portsystem} {
-                ui_error "Line $lineno repeats PortSystem information"
+                ui_error "Line $lineno repeats PortSystem declaration"
                 incr errors
             }
             regexp {PortSystem\s+([0-9.]+)} $line -> portsystem
@@ -204,16 +201,17 @@ proc portlint::lint_main {args} {
             set require_after "PortSystem"
         }
         if {[string match "PortGroup*" $line]} {
-            regexp {PortGroup\s+([a-z0-9]+)\s+([0-9.]+)} $line -> portgroup portgroupversion
+            regexp {PortGroup\s+([A-Za-z0-9_]+)\s+([0-9.]+)} $line -> portgroup portgroupversion
             if {![info exists portgroup]} {
                 ui_error "Line $lineno has unrecognized PortGroup"
                 incr errors
-            }
-            if {[info exists portgroups($portgroup)]} {
-                ui_error "Line $lineno repeats PortGroup information"
-                incr errors
             } else {
-                set portgroups($portgroup) $portgroupversion
+                if {[info exists portgroups($portgroup)]} {
+                    ui_error "Line $lineno repeats inclusion of PortGroup $portgroup"
+                    incr errors
+                } else {
+                    set portgroups($portgroup) $portgroupversion
+                }
             }
             set seen_portgroup true
             set require_blank true
@@ -227,7 +225,7 @@ proc portlint::lint_main {args} {
         if {[string match "long_description*" $line]} {
             set in_description true
         }
-        if {$in_description && ([string range $line end end] != "\\")} {
+        if {$in_description && ([string range $line end end] ne "\\")} {
             set in_description false
             #set require_blank true
             #set require_after "long_description"
@@ -244,7 +242,7 @@ proc portlint::lint_main {args} {
         
         if {[string match "platform\[ \t\]*" $line]} {
             regexp {platform\s+(?:\w+\s+(?:\w+\s+)?)?(\w+)} $line -> platform_arch
-            if {$platform_arch == "ppc"} {
+            if {$platform_arch eq "ppc"} {
                 ui_error "Arch 'ppc' in platform on line $lineno should be 'powerpc'"
                 incr errors
             }
@@ -269,8 +267,9 @@ proc portlint::lint_main {args} {
             }
     
             if {!$hashline
-                    && ![regexp {^PortSystem|^PortGroup|^version} $line]
-                    && ![regexp {^[a-z0-9]+\.setup} $line]
+                    && ![regexp {^\s*PortSystem|^\s*PortGroup|^\s*version} $line]
+                    && ![regexp {^\s*[A-Za-z0-9_]+\.setup} $line]
+                    && ![regexp {^\s*license} $line]
                     && [string first [option version] $line] != -1} {
                 ui_warn "Line $lineno seems to hardcode the version number, consider using \${version} instead"
                 incr warnings
@@ -283,7 +282,16 @@ proc portlint::lint_main {args} {
                     set hashline false
             }
         }
-            
+
+        # Check for hardcoded paths
+        if {!$hashline
+                && $name ne "MacPorts"
+                && [string match "*/opt/local*" $line]
+                && ![regexp {^\s*reinplace} $line]
+                && ![regexp {^\s*system.*\Wsed\W} $line]} {
+            ui_error "Line $lineno hardcodes /opt/local, use \${prefix} instead"
+        }
+
         ### TODO: more checks to Portfile syntax
 
         incr lineno
@@ -291,15 +299,13 @@ proc portlint::lint_main {args} {
 
     ###################################################################
 
-    global os.platform os.arch os.version
-    global version revision epoch
+    global os.platform os.arch os.version version revision epoch \
+           description long_description platforms categories all_variants \
+           maintainers license homepage master_sites checksums patchfiles \
+           depends_fetch depends_extract depends_lib depends_build \
+           depends_run distfiles fetch.type lint_portsystem lint_platforms \
+           lint_required lint_optional
     set portarch [get_canonical_archs]
-    global description long_description platforms categories all_variants
-    global maintainers license homepage master_sites checksums patchfiles
-    global depends_fetch depends_extract depends_lib depends_build depends_run distfiles fetch.type
-    
-    global lint_portsystem lint_platforms
-    global lint_required lint_optional
 
     if (!$seen_portsystem) {
         ui_error "Didn't find PortSystem specification"
@@ -325,7 +331,7 @@ proc portlint::lint_main {args} {
 
     foreach req_var $lint_required {
 
-        if {$req_var == "master_sites"} {
+        if {$req_var eq "master_sites"} {
             if {${fetch.type} != "standard"} {
                 ui_info "OK: $req_var not required for fetch.type ${fetch.type}"
                 continue
@@ -392,7 +398,7 @@ proc portlint::lint_main {args} {
     foreach variant $all_variants {
         set variantname [ditem_key $variant name] 
         set variantdesc [lindex [ditem_key $variant description] 0]
-        if {![info exists variantname] || $variantname == ""} {
+        if {![info exists variantname] || $variantname eq ""} {
             ui_error "Variant number $variantnumber does not have a name"
             incr errors
         } else {
@@ -405,18 +411,18 @@ proc portlint::lint_main {args} {
                 set name_ok false
             }
 
-            if {![info exists variantdesc] || $variantdesc == ""} {
+            if {![info exists variantdesc] || $variantdesc eq ""} {
                 # don't warn about missing descriptions for global variants
                 if {[lsearch -exact $local_variants $variantname] != -1 &&
-                    [variant_desc $porturl $variantname] == ""} {
+                    [variant_desc $porturl $variantname] eq ""} {
                     ui_warn "Variant $variantname does not have a description"
                     incr warnings
                     set desc_ok false
-                } elseif {$variantdesc == ""} {
+                } elseif {$variantdesc eq ""} {
                     set variantdesc "(pre-defined variant)"
                 }
             } else {
-                if {[variant_desc $porturl $variantname] != ""} {
+                if {[variant_desc $porturl $variantname] ne ""} {
                     ui_warn "Variant $variantname overrides global description"
                     incr warnings
                 }
@@ -461,7 +467,7 @@ proc portlint::lint_main {args} {
             ui_debug "$errorInfo"
             continue
         }
-        if {$res == ""} {
+        if {$res eq ""} {
             ui_error "Unknown dependency: $dep"
             incr errors
         } else {
@@ -500,19 +506,19 @@ proc portlint::lint_main {args} {
                 $addr == "openmaintainer@macports.org"} {
             ui_warn "Using full email address for no/open maintainer"
             incr warnings
-        } elseif [regexp "^(.+)@macports.org$" $addr -> name] {
-            ui_warn "Maintainer email address for $name includes @macports.org"
+        } elseif {[regexp "^(.+)@macports.org$" $addr -> localpart]} {
+            ui_warn "Maintainer email address for $localpart includes @macports.org"
             incr warnings
         } elseif {$addr == "darwinports@opendarwin.org"} {
             ui_warn "Using legacy email address for no/open maintainer"
             incr warnings
-        } elseif [regexp "^(.+)@(.+)$" $addr -> localpart domain] {
+        } elseif {[regexp "^(.+)@(.+)$" $addr -> localpart domain]} {
             ui_warn "Maintainer email address should be obfuscated as $domain:$localpart"
             incr warnings
         }
     }
 
-    if {$license == "unknown"} {
+    if {$license eq "unknown"} {
         ui_warn "no license set"
         incr warnings
     } else {
@@ -538,28 +544,24 @@ proc portlint::lint_main {args} {
                         # if the last character of license name is a number or plus sign
                         # then a hyphen is missing
                         set license_end [string index $subtest end]
-                        if {[string equal "+" $license_end] || [string is integer -strict $license_end]} {
+                        if {"+" eq $license_end || [string is integer -strict $license_end]} {
                             ui_error "invalid license '${test}': missing hyphen before version"
                         }
                     }
                 }
             }
 
-            # BSD-2 => BSD
             if {[string equal -nocase "BSD-2" $test]} {
+                # BSD-2 => BSD
                 ui_error "Invalid license '${test}': use BSD instead"
-            }
-    
-            # BSD-3 => BSD
-            if {[string equal -nocase "BSD-3" $test]} {
+            } elseif {[string equal -nocase "BSD-3" $test]} {
+                # BSD-3 => BSD
                 ui_error "Invalid license '${test}': use BSD instead"
-            }
-    
-            # BSD-4 => BSD-old
-            if {[string equal -nocase "BSD-4" $test]} {
+            } elseif {[string equal -nocase "BSD-4" $test]} {
+                # BSD-4 => BSD-old
                 ui_error "Invalid license '${test}': use BSD-old instead"
             }
-    
+
             set prev $test
         }
 
@@ -596,7 +598,7 @@ proc portlint::lint_main {args} {
         set refcount  [lindex [set ${deprecated_options_name}($option)] 1]
 
         if {$refcount > 0} {
-            if {$newoption != ""} {
+            if {$newoption ne ""} {
                 ui_warn "Using deprecated option '$option', superseded by '$newoption'"
             } else {
                 ui_warn "Using deprecated option '$option'"
@@ -615,32 +617,6 @@ proc portlint::lint_main {args} {
 
     ###################################################################
 
-    set svn_cmd ""
-    catch {set svn_cmd [findBinary svn]}
-    if {$svn_cmd != "" && ([file exists $portpath/.svn] || ![catch {exec $svn_cmd info $portpath > /dev/null 2>@1}])} {
-        ui_debug "Checking svn properties"
-        if [catch {exec $svn_cmd propget svn:keywords $portfile 2>@1} output] {
-            ui_warn "Unable to check for svn:keywords property: $output"
-        } else {
-            ui_debug "Property svn:keywords is \"$output\", should be \"Id\""
-            if {$output != "Id"} {
-                ui_error "Missing subversion property on Portfile, please execute: svn ps svn:keywords Id Portfile"
-                incr errors
-            }
-        }
-        if [catch {exec $svn_cmd propget svn:eol-style $portfile 2>@1} output] {
-            ui_warn "Unable to check for svn:eol-style property: $output"
-        } else {
-            ui_debug "Property svn:eol-style is \"$output\", should be \"native\""
-            if {$output != "native"} {
-                ui_error "Missing subversion property on Portfile, please execute: svn ps svn:eol-style native Portfile"
-                incr errors
-            }
-        }
-    }
-
-    ###################################################################
-
     ui_notice "$UI_PREFIX [format [msgcat::mc "%d errors and %d warnings found."] $errors $warnings]"
 
     return {$errors > 0}
