@@ -1,9 +1,10 @@
 # receipt_flat.tcl
-# $Id: receipt_flat.tcl 41855 2008-11-11 09:27:17Z ryandesign@macports.org $
+# $Id$
 #
+# Copyright (c) 2005, 2007-2012 The MacPorts Project
 # Copyright (c) 2004 Will Barton <wbb4@opendarwin.org>
 # Copyright (c) 2004 Paul Guyot, The MacPorts Project.
-# Copyright (c) 2002 Apple Computer, Inc.
+# Copyright (c) 2002 Apple Inc.
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -14,7 +15,7 @@
 # 2. Redistributions in binary form must reproduce the above copyright
 #    notice, this list of conditions and the following disclaimer in the
 #    documentation and/or other materials provided with the distribution.
-# 3. Neither the name of Apple Computer, Inc. nor the names of its contributors
+# 3. Neither the name of Apple Inc. nor the names of its contributors
 #    may be used to endorse or promote products derived from this software
 #    without specific prior written permission.
 # 
@@ -43,6 +44,8 @@ namespace eval receipt_flat {
 
 # receipt_lastref is the last attributed index of receipts.
 variable receipt_lastref -1
+# maps port name,version,variants to the corresponding receipt ref
+variable ref_index
 
 ##
 #
@@ -72,7 +75,7 @@ proc get_head_entry_receipt_path {portname portversion} {
 
     # regex match case
     if {$portversion == 0} {
-	set x [glob -nocomplain [file join ${macports::registry.path} receipts ${portname}-*]]
+	set x [glob -nocomplain -directory [::file join ${macports::registry.path} receipts] -- ${portname}-*]
 	if {[string length $x]} {
 	    set matchfile [lindex $x 0]
 		# Remove trailing .bz2, if any.
@@ -81,15 +84,15 @@ proc get_head_entry_receipt_path {portname portversion} {
 	    set matchfile ""
 	}
     } else {
-	set matchfile [file join ${macports::registry.path} receipts ${portname}-${portversion}]
+	set matchfile [::file join ${macports::registry.path} receipts ${portname}-${portversion}]
     }
 
     # Might as well bail out early if no file to match
-    if {![string length $matchfile]} {
+    if {$matchfile eq ""} {
 		return ""
     }
 
-    if {[file exists $matchfile] || [file exists ${matchfile}.bz2]} {
+    if {[::file exists $matchfile] || [::file exists ${matchfile}.bz2]} {
 		return $matchfile
     }
     return ""
@@ -98,19 +101,24 @@ proc get_head_entry_receipt_path {portname portversion} {
 ##
 #
 # Open an existing entry and return its reference number.
-proc open_entry {name {version ""} {revision 0} {variants ""}} {
-	global macports::registry.installtype
+proc open_entry {name {version ""} {revision 0} {variants ""} {epoch ""}} {
 	global macports::registry.path
+	variable ref_index
+	
+	# if this entry is already open, just return the reference
+	if {[info exists ref_index($name,$version,$revision,$variants)]} {
+	    return $ref_index($name,$version,$revision,$variants)
+	}
 
-	set receipt_path [file join ${macports::registry.path} receipts ${name}]
+	set receipt_path [::file join ${macports::registry.path} receipts ${name}]
 
 	# If the receipt path ${name} doesn't exist, then the receipt probably is
 	# in the old HEAD format.
-	if { ![file isdirectory $receipt_path] } {
+	if { ![::file isdirectory $receipt_path] } {
 		set receipt_file [get_head_entry_receipt_path $name $version]
 		
-		if {![string length $receipt_file]} {
-			if { $version != "" } {
+		if {$receipt_file eq ""} {
+			if { $version ne "" } {
 				return -code error "Registry error: ${name} @${version}_${revision}${variants} not registered as installed."
 			} else {
 				return -code error "Registry error: ${name} not registered as installed."
@@ -118,19 +126,18 @@ proc open_entry {name {version ""} {revision 0} {variants ""}} {
 		}
 		
 		# Extract the version from the path.
-		if { $version == "" } {
-			set theFileName [file tail $receipt_file]
+		if { $version eq "" } {
+			set theFileName [::file tail $receipt_file]
 			regexp "^$name-(.*)\$" $theFileName match version
 		}
 	} else {
-		# If version wasn't specified, find out the version number.  This will
-		# depend on which installtype mode we're in, "direct" or "image"	
-		if { $version == "" } {
-			# xxx: If we're in image mode, we really should have had the 
+		# If version wasn't specified, find out the version number.
+		if { $version eq "" } {
+			# xxx: We really should have had the 
 			# version given to us.  How should we handle this?
-			set x [glob -nocomplain [file join ${receipt_path} *]]
+			set x [glob -nocomplain -directory ${receipt_path} *]
 			if { [string length $x] } {
-				set v [lindex [file split [lindex $x 0]] end]
+				set v [lindex [::file split [lindex $x 0]] end]
 				regexp {([-_a-zA-Z0-9\.]+)_([0-9]*)([+-_a-zA-Z0-9]*)$} $v match version revision variants
 			} else {
 				return -code error "Registry error: ${name} not registered as installed."
@@ -141,15 +148,15 @@ proc open_entry {name {version ""} {revision 0} {variants ""}} {
 			return -code error "Registry error: ${name} @${version}_${revision}${variants} not registered as installed."
 		}
 	
-		set receipt_path [file join ${macports::registry.path} receipts ${name} ${version}_${revision}${variants}]
+		set receipt_path [::file join ${macports::registry.path} receipts ${name} ${version}_${revision}${variants}]
 	
-		set receipt_file [file join ${receipt_path} receipt]
+		set receipt_file [::file join ${receipt_path} receipt]
 	}
 
-	if { [file exists ${receipt_file}.bz2] && [file exists ${registry::autoconf::bzip2_path}] } {
+	if { [::file exists ${receipt_file}.bz2] && [::file exists ${registry::autoconf::bzip2_path}] } {
 		set receipt_file ${receipt_file}.bz2
 		set receipt_contents [exec ${registry::autoconf::bzip2_path} -d -c ${receipt_file}]
-	} elseif { [file exists ${receipt_file}] } {
+	} elseif { [::file exists ${receipt_file}] } {
 		set receipt_handle [open ${receipt_file} r]
 		set receipt_contents [read $receipt_handle]
 		close $receipt_handle
@@ -170,9 +177,9 @@ proc open_entry {name {version ""} {revision 0} {variants ""}} {
 		convert_entry_from_HEAD $name $version $revision $variants $receipt_contents $ref
 		
 		# move the old receipt
-		set convertedDirPath [file join ${macports::registry.path} receipts_converted]
+		set convertedDirPath [::file join ${macports::registry.path} receipts_converted]
 		file mkdir $convertedDirPath
-		file rename $receipt_file $convertedDirPath
+		file rename -- $receipt_file $convertedDirPath
 	} elseif {[string match "# Version: *" $receipt_contents]} {
 		# This is new format
 		if {![string match "# Version: 1.0*" $receipt_contents]} {
@@ -181,7 +188,11 @@ proc open_entry {name {version ""} {revision 0} {variants ""}} {
 
 		# Remove any line starting with #
 		while {[regexp "(^|\n)#.*\n(.*)\$" $receipt_contents match foo receipt_contents]} {}
-		array set receipt_$ref $receipt_contents
+		if {[catch {array set receipt_$ref $receipt_contents} rcpterr]} {
+			ui_error "Malformed receipt for ${name} @${version}_${revision}${variants}"
+			ui_error "receipt_contents = '$receipt_contents'"
+			error $rcpterr
+		}
 	} else {
 		# This is old Images format
 
@@ -191,6 +202,8 @@ proc open_entry {name {version ""} {revision 0} {variants ""}} {
 		}
 	}
 	
+	set ref_index($name,$version,$revision,$variants) $ref
+	
 	return $ref
 }
 
@@ -261,15 +274,15 @@ proc convert_entry_from_HEAD {name version revision variants receipt_contents re
 	foreach file $contents {
 		if {[llength $file]} {
 			set theFilePath [lindex $file 0]
-			if {[file isfile $theFilePath]} {
-				set previousPort [file_registered $theFilePath]
+			if {[::file isfile $theFilePath]} {
+				set previousPort [::file_registered $theFilePath]
 				if {$previousPort != 0} {
 					ui_warn "Conflict detected for file $theFilePath between $previousPort and $name."
 				}
 				if {[catch {register_file $theFilePath $name}]} {
 					ui_warn "An error occurred while adding $theFilePath to the file_map database."
 				}
-			} elseif {![file exists $theFilePath]} {
+			} elseif {![::file exists $theFilePath]} {
 				ui_warn "Port $name refers to $theFilePath which doesn't exist."
 			}
 			lappend theActualContents $file
@@ -282,6 +295,7 @@ proc convert_entry_from_HEAD {name version revision variants receipt_contents re
 
 	# Save the file_map afterwards
 	write_file_map
+	close_file_map
 	
 	# Save the entry to new format.
 	write_entry $ref $name $version $revision $variants
@@ -296,15 +310,14 @@ proc convert_entry_from_HEAD {name version revision variants receipt_contents re
 # version			the version of the port.
 # variants			the variants of the port.
 proc write_entry {ref name version {revision 0} {variants ""}} {
-	global macports::registry.installtype
 	variable receipt_$ref
 
 	set receipt_contents [array get receipt_$ref]
 
-	set receipt_path [file join ${macports::registry.path} receipts ${name} ${version}_${revision}${variants}]
-	set receipt_file [file join ${receipt_path} receipt]
+	set receipt_path [::file join ${macports::registry.path} receipts ${name} ${version}_${revision}${variants}]
+	set receipt_file [::file join ${receipt_path} receipt]
 
-	if { ![file isdirectory ${receipt_path}] } {
+	if { ![::file isdirectory ${receipt_path}] } {
 		file mkdir ${receipt_path}
 	}
 
@@ -313,15 +326,15 @@ proc write_entry {ref name version {revision 0} {variants ""}} {
 	puts $receipt_handle $receipt_contents
 	close $receipt_handle
 
-	if { [file exists ${receipt_file}] } {
-		system "rm -rf ${receipt_file}"
-	} elseif { [file exists ${receipt_file}.bz2] } {
-		system "rm -rf ${receipt_file}.bz2"
+	if { [::file exists ${receipt_file}] } {
+		file delete -force -- "${receipt_file}"
+	} elseif { [::file exists ${receipt_file}.bz2] } {
+		file delete -force -- "${receipt_file}.bz2"
 	}
 
-	system "mv ${receipt_file}.tmp ${receipt_file}"
+	file rename -force -- "${receipt_file}.tmp" "${receipt_file}"
 
-	if { [file exists ${receipt_file}] && [file exists ${registry::autoconf::bzip2_path}] && ![info exists registry.nobzip] } {
+	if { [::file exists ${receipt_file}] && [::file exists ${registry::autoconf::bzip2_path}] && ![info exists registry.nobzip] } {
 		system "${registry::autoconf::bzip2_path} -f ${receipt_file}"
 	}
 
@@ -331,16 +344,26 @@ proc write_entry {ref name version {revision 0} {variants ""}} {
 # Check to see if an entry exists
 proc entry_exists {name version {revision 0} {variants ""}} {
 	global macports::registry.path
-	variable receipt_handle 
-	variable receipt_file 
-	variable receipt_path
 
-	set receipt_path [file join ${macports::registry.path} receipts ${name} ${version}_${revision}${variants}]
-	set receipt_file [file join ${receipt_path} receipt]
+	set receipt_path [::file join ${macports::registry.path} receipts ${name} ${version}_${revision}${variants}]
+	set receipt_file [::file join ${receipt_path} receipt]
 
-	if { [file exists $receipt_file] } {
+	if { [::file exists $receipt_file] } {
 		return 1
-	} elseif { [file exists ${receipt_file}.bz2] } {
+	} elseif { [::file exists ${receipt_file}.bz2] } {
+		return 1
+	}
+
+	return 0
+}
+
+# Check to see if any entry exists for the given port name
+proc entry_exists_for_name {name} {
+	global macports::registry.path
+
+	set receipt_path [::file join ${macports::registry.path} receipts ${name}]
+
+	if {[llength [glob -nocomplain -directory $receipt_path */receipt{,.bz2}]] > 0} {
 		return 1
 	}
 
@@ -384,19 +407,28 @@ proc property_retrieve {ref property} {
 # Delete an entry
 proc delete_entry {name version {revision 0} {variants ""}} {
 	global macports::registry.path
+	variable ref_index
+	
+	# if the entry is loaded, purge it
+	if {[info exists ref_index($name,$version,$revision,$variants)]} {
+	    set ref $ref_index($name,$version,$revision,$variants)
+	    variable receipt_${ref}
+	    array unset receipt_${ref}
+	    array unset ref_index "$name,$version,$revision,$variants"
+	}
 
-	set receipt_path [file join ${macports::registry.path} receipts ${name} ${version}_${revision}${variants}]
-	if { [file exists ${receipt_path}] } {
+	set receipt_path [::file join ${macports::registry.path} receipts ${name} ${version}_${revision}${variants}]
+	if { [::file exists ${receipt_path}] } {
 		# remove port receipt directory
 		ui_debug "deleting directory: ${receipt_path}"
-		file delete -force ${receipt_path}
+		file delete -force -- ${receipt_path}
 		# remove port receipt parent directory (if empty)
-		set receipt_dir [file join ${macports::registry.path} receipts ${name}]
-		if { [file isdirectory ${receipt_dir}] } {
+		set receipt_dir [::file join ${macports::registry.path} receipts ${name}]
+		if { [::file isdirectory ${receipt_dir}] } {
 			# 0 item means empty.
 			if { [llength [readdir ${receipt_dir}]] == 0 } {
 				ui_debug "deleting directory: ${receipt_dir}"
-				file delete -force ${receipt_dir}
+				file delete -force -- ${receipt_dir}
 			} else {
 				ui_debug "${receipt_dir} is not empty"
 			}
@@ -417,12 +449,12 @@ proc delete_entry {name version {revision 0} {variants ""}} {
 proc installed {{name ""} {version ""}} {
 	global macports::registry.path
 
-	set query_path [file join ${macports::registry.path} receipts]
+	set query_path [::file join ${macports::registry.path} receipts]
 	
-	if { $name == "" } {
-		set query_path [file join ${query_path} *]
-		if { $version == "" } {
-			set query_path [file join ${query_path} *]
+	if { $name eq "" } {
+		set query_path [::file join ${query_path} *]
+		if { $version eq "" } {
+			set query_path [::file join ${query_path} *]
 		}
 		# [PG] Huh?
 	} else {
@@ -430,29 +462,32 @@ proc installed {{name ""} {version ""}} {
 	    # returned with the correct case even if it's wrong when given. To get the
 	    # correct case on a case-insensitive FS, we have to list the directory and
 	    # compare against each entry.
-	    set name_path [file join ${query_path} *]
+	    set name_path [::file join ${query_path} *]
 	    set name_entries [glob -nocomplain -types d ${name_path}]
 	    foreach entry $name_entries {
-	        set basename [file tail $entry]
+	        set basename [::file tail $entry]
 	        if {[string equal -nocase $basename $name]} {
 	            set name $basename
 	            break
 	        }
 	    }
-		set query_path [file join ${query_path} ${name}]
-		if { $version != "" } {
-			set query_path [file join ${query_path} ${version}]
+		set query_path [::file join ${query_path} ${name}]
+		if { $version ne "" } {
+			set query_path [::file join ${query_path} ${version}]
 		} else {
-			set query_path [file join ${query_path} *]
+			set query_path [::file join ${query_path} *]
 		}
 	}
 
 	set x [glob -nocomplain -types d ${query_path}]
 	set rlist [list]
 	foreach p $x {
+		if {![::file isfile [::file join $p receipt.bz2]] && ![::file isfile [::file join $p receipt]]} {
+			continue
+		}
 		set plist [list]
-		regexp {([-_a-zA-Z0-9\.]+)_([0-9]*)([+-_a-zA-Z0-9]*)$} [lindex [file split $p] end] match version revision variants
-		lappend plist [lindex [file split $p] end-1]
+		regexp {([-_a-zA-Z0-9\.]+)_([0-9]*)([+-_a-zA-Z0-9]*)$} [lindex [::file split $p] end] match version revision variants
+		lappend plist [lindex [::file split $p] end-1]
 		lappend plist $version
 		lappend plist $revision
 		lappend plist $variants
@@ -460,18 +495,18 @@ proc installed {{name ""} {version ""}} {
 	}
 
 	# append the ports in old HEAD format.
-	if { $name == "" } {
-		set query_path [file join ${macports::registry.path} receipts *]
+	if { $name eq "" } {
+		set query_path [::file join ${macports::registry.path} receipts *]
 	} else {
-		set query_path [file join ${macports::registry.path} receipts ${name}-*]
+		set query_path [::file join ${macports::registry.path} receipts ${name}-*]
 	}
     set receiptglob [glob -nocomplain -types f ${query_path}]
     foreach receipt_file $receiptglob {
-		set theFileName [file tail $receipt_file]
+		set theFileName [::file tail $receipt_file]
 
     	# Remark: these regexes do not always work.
    		set theName ""
-    	if { $name == "" } {
+    	if { $name eq "" } {
 			regexp {^(.*)-(.*)$} $theFileName match theName version
     	} else {
 			regexp "^($name)-(.*)\$" $theFileName match theName version
@@ -504,14 +539,14 @@ proc open_file_map {{readonly 0}} {
 	global macports::registry.path
 	variable file_map
 
-	set receipt_path [file join ${macports::registry.path} receipts]
-	set map_file [file join ${receipt_path} file_map]
+	set receipt_path [::file join ${macports::registry.path} receipts]
+	set map_file [::file join ${receipt_path} file_map]
 
 	# Don't reopen it (it actually would deadlock us), unless it was open r/o.
 	# and we want it r/w.
 	if { [info exists file_map] } {
 		if { $readonly == 0 } {
-			if {[filemap isreadonly file_map]} {
+			if {[::filemap isreadonly file_map]} {
 				filemap close file_map
 				filemap open file_map ${map_file}.db
 			}
@@ -521,11 +556,11 @@ proc open_file_map {{readonly 0}} {
 
 	set old_filemap [list]
 
-	if { ![file exists ${map_file}.db] } {
+	if { ![::file exists ${map_file}.db] } {
 		# Convert to new format
-		if { [file exists ${map_file}.bz2] && [file exists ${registry::autoconf::bzip2_path}] } {
+		if { [::file exists ${map_file}.bz2] && [::file exists ${registry::autoconf::bzip2_path}] } {
 			set old_filemap [exec ${registry::autoconf::bzip2_path} -d -c ${map_file}.bz2]
-		} elseif { [file exists $map_file] } {		
+		} elseif { [::file exists $map_file] } {		
 			set map_handle [open ${map_file} r]
 			set old_filemap [read $map_handle]
 			close $map_handle
@@ -573,8 +608,8 @@ proc file_registered {file} {
 
 	open_file_map 1
 
-	if {[filemap exists file_map $file]} {
-		return [filemap get file_map $file]
+	if {[::filemap exists file_map $file]} {
+		return [::filemap get file_map $file]
 	} else {
 		return 0
 	}
@@ -595,7 +630,7 @@ proc port_registered {name} {
 
 	open_file_map 1
 
-	set files [filemap list file_map $name]
+	set files [::filemap list file_map $name]
 
 	if { [llength $files] > 0 } {
 		return $files
@@ -606,7 +641,7 @@ proc port_registered {name} {
 			# Convert the port and retry.
 			open_entry $name
 			
-			set files [filemap list file_map $name]
+			set files [::filemap list file_map $name]
 			
 			return $files
 		} else {
@@ -627,7 +662,7 @@ proc register_file {file port} {
 
 	open_file_map
 
-	if { [file type $file] == "link" } {
+	if { [::file type $file] eq "link" } {
 		ui_debug "Adding link to file_map: $file for: $port"
 	} else {
 		ui_debug "Adding file to file_map: $file for: $port"
@@ -649,7 +684,7 @@ proc register_bulk_files {files port} {
 
 	foreach f $files {
 		set file [lindex $f 0]
-		if { [file type $file] == "link" } {
+		if { [::file type $file] eq "link" } {
 			ui_debug "Adding link to file_map: $file for: $port"
 		} else {
 			ui_debug "Adding file to file_map: $file for: $port"
@@ -690,18 +725,31 @@ proc write_file_map {args} {
 	return 1
 }
 
+##
+# close the file map.
+# important to do this so the lock is released.
+# do not do anything if the file map wasn't open.
+#
+proc close_file_map {args} {
+	variable file_map
+
+	if { [info exists file_map] } {
+		filemap close file_map
+	}
+}
+
 # Dependency Map Code
 proc open_dep_map {args} {
 	global macports::registry.path
 	variable dep_map
 
-	set receipt_path [file join ${macports::registry.path} receipts]
+	set receipt_path [::file join ${macports::registry.path} receipts]
 
-	set map_file [file join ${receipt_path} dep_map]
+	set map_file [::file join ${receipt_path} dep_map]
 
-	if { [file exists ${map_file}.bz2] && [file exists ${registry::autoconf::bzip2_path}] } {
+	if { [::file exists ${map_file}.bz2] && [::file exists ${registry::autoconf::bzip2_path}] } {
 		set dep_map [exec ${registry::autoconf::bzip2_path} -d -c ${map_file}.bz2]
-	} elseif { [file exists ${map_file}] } {
+	} elseif { [::file exists ${map_file}] } {
 		set map_handle [open ${map_file} r]
 		set dep_map [read $map_handle]
 		close $map_handle
@@ -714,7 +762,7 @@ proc open_dep_map {args} {
 }
 
 # List all ports this one depends on
-proc list_depends {name} {
+proc list_depends {name {version ""} {revision ""} {variants 0}} {
 	variable dep_map
 	if { [llength $dep_map] < 1 && [info exists dep_map] } {
 		open_dep_map
@@ -729,7 +777,7 @@ proc list_depends {name} {
 }
 
 # List all the ports that depend on this port
-proc list_dependents {name} {
+proc list_dependents {name version revision variants} {
 	variable dep_map
 	if { [llength $dep_map] < 1 && [info exists dep_map] } {
 		open_dep_map
@@ -780,7 +828,7 @@ proc clean_dep_map {args} {
     set dep_map $new_map
     
     set newlen [llength $dep_map]
-    set diff [expr $oldlen - $newlen]
+    set diff [expr {$oldlen - $newlen}]
     ui_debug "New dep_map has $newlen entries"
     ui_info "Removed $diff duplicate entries from the dependency map"
 }
@@ -789,20 +837,20 @@ proc write_dep_map {args} {
 	global macports::registry.path
 	variable dep_map
 
-	set receipt_path [file join ${macports::registry.path} receipts]
+	set receipt_path [::file join ${macports::registry.path} receipts]
 
-	set map_file [file join ${receipt_path} dep_map]
+	set map_file [::file join ${receipt_path} dep_map]
 
 	set map_handle [open ${map_file}.tmp w 0644]
 	puts $map_handle $dep_map
 	close $map_handle
 
     # don't both checking for presence, file delete doesn't error if file doesn't exist
-    file delete ${map_file} ${map_file}.bz2
+    file delete -- ${map_file} ${map_file}.bz2
 
-    file rename ${map_file}.tmp ${map_file}
+    file rename -- ${map_file}.tmp ${map_file}
 
-	if { [file exists ${map_file}] && [file exists ${registry::autoconf::bzip2_path}] && ![info exists registry.nobzip] } {
+	if { [::file exists ${map_file}] && [::file exists ${registry::autoconf::bzip2_path}] && ![info exists registry.nobzip] } {
 		system "${registry::autoconf::bzip2_path} -f ${map_file}"
 	}
 
